{% extends "wiki/_base.html" %}

{% from "macros/box.html" import m_box_content %}

{% block content %}
  {% call m_box_content(_("(source)")) %}
    <pre>{{ page.body_src }}</pre>
  {% endcall %}
{% endblock %}
