// Community header
table.community-list tr {
  td {
    vertical-align: middle;

    &.logo {
      width: 65px !important;
      height: 65px !important;
      
      img { border-radius: 10%; }      
    }

    a {
      text-decoration: none;
      color: #555; /* TODO: global font color */
    }

    .profile {
      margin: 10px 0 10px 10px;
    }

    .name {
      margin: 2px 0;
      font-size: 1.5em;
      font-weight: bold;
    }

    .description {
      font-style: italic;
    }
  }

  &:last-child td {
    border-bottom: 1px solid #dddddd;
  }
}
