{% extends "social/base.html" %}
{% from "macros/box.html" import m_box %}
{% from "macros/activity.html" import m_activities %}

{% block content %}
  {% call m_box(_("Activity stream"), color="blue") %}
    {{ m_activities(activity_entries) }}
  {% endcall %}
{% endblock %}

{% block sidebar %}
  {% include "social/_sidebar.html" %}
{%- endblock %}
